Package: chngpt
LazyLoad: yes
LazyData: yes
Version: 2024.11-15
Title: Estimation and Hypothesis Testing for Threshold Regression
Authors@R: c(person("Youyi", "Fong", role = "cre", email = "youyifong@gmail.com"),
    person("Qianqian", "Chen", role = "aut", email = "qc25@uw.edu"),
    person("Shuangcheng", "Hua", role = "aut", email = "shuangch.hua@gmail.com"),
    person("Hyunju", "Son", role = "aut", email = "hson@uw.edu"),
    person("Adam", "Elder", role = "aut", email = "eldera3@uw.edu"),
    person("Tao", "Yang", role = "aut", email = "tyang23@fredhutch.org"),
    person("Zonglin", "He", role = "aut", email = "nkhezl@gmail.com"),
    person("Simone", "Giannerini", role = "aut", email = "simone.giannerini@unibo.it")
    )
Depends: R (>= 3.6)
Suggests: R.rsp, RUnit, mvtnorm
Imports: survival, splines, kyotil (>= 2020.10-12), boot, MASS,
        methods, lme4, parallel, RhpcBLASctl
VignetteBuilder: R.rsp
Description: Threshold regression models are also called two-phase regression, broken-stick regression, split-point regression, structural change models, and regression kink models, with and without interaction terms. Methods for both continuous and discontinuous threshold models are included, but the support for the former is much greater. This package is described in Fong, Huang, Gilbert and Permar (2017) <DOI:10.1186/s12859-017-1863-x> and the package vignette.
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2024-11-12 20:15:53 UTC; Youyi
Author: Youyi Fong [cre],
  Qianqian Chen [aut],
  Shuangcheng Hua [aut],
  Hyunju Son [aut],
  Adam Elder [aut],
  Tao Yang [aut],
  Zonglin He [aut],
  Simone Giannerini [aut]
Maintainer: Youyi Fong <youyifong@gmail.com>
Repository: CRAN
Date/Publication: 2024-11-12 20:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 06:27:04 UTC; unix
Archs: chngpt.so.dSYM
